@@ -550,6 +550,9 @@ class TestPloneApiContent(unittest.TestCase):
                 container['about']['link-to-blog-1-1'] == linktoblog11)
         assert 'link-to-blog' not in container.keys()
 
+    def test_rename_same_id(self):
+        api.content.rename(obj=self.contact, new_id=self.contact.getId())
+
     def test_rename_same_folder(self):
         # When renaming a folderish item with safe_id=True, and there is
         # already an existing folderish item with that id, it should choose
@@ -732,6 +735,10 @@ class TestPloneApiContent(unittest.TestCase):
         self.assertNotIn('copy_of_about', container)
         self.assertNotIn('about', container['events'])
 
+    def test_delete_no_objs(self):
+        # Check that we allow passing in an empty list of objects.
+        api.content.delete(obj=None, objects=[])
+
     def test_delete_ignore_linkintegrity(self):
         """Test deleting a content item with a link pointed at it."""
         self._set_text(self.team, '<a href="contact">contact</a>')
