@@ -79,7 +79,7 @@ class TestPloneApiUser(unittest.TestCase):
         user = api.user.create(
             username='chuck',
             password='secret',
-            properties={'email': 'chuck@norris.org'}
+            properties={'email': 'chuck@norris.org'},
         )
 
         self.assertEqual(user.getProperty('email'), 'chuck@norris.org')
@@ -92,7 +92,7 @@ class TestPloneApiUser(unittest.TestCase):
 
         user = api.user.create(
             email='chuck@norris.org',
-            password='secret'
+            password='secret',
         )
 
         self.assertEqual(user.getUserName(), 'chuck@norris.org')
@@ -136,9 +136,9 @@ class TestPloneApiUser(unittest.TestCase):
             email='chuck@norris.org',
             password='secret',
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(user=user),
-            ['Member', 'Authenticated', ]
+            ['Member', 'Authenticated', ],
         )
 
     def test_create_specified_roles(self):
@@ -147,11 +147,11 @@ class TestPloneApiUser(unittest.TestCase):
             username='chuck',
             email='chuck@norris.org',
             password='secret',
-            roles=['Reviewer', 'Editor']
+            roles=['Reviewer', 'Editor'],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(user=user),
-            ['Reviewer', 'Authenticated', 'Editor']
+            ['Reviewer', 'Authenticated', 'Editor'],
         )
 
     def test_create_no_roles(self):
@@ -160,11 +160,11 @@ class TestPloneApiUser(unittest.TestCase):
             username='chuck',
             email='chuck@norris.org',
             password='secret',
-            roles=[]
+            roles=[],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(user=user),
-            ['Authenticated', ]
+            ['Authenticated', ],
         )
 
     def test_get_constraints(self):
@@ -187,7 +187,10 @@ class TestPloneApiUser(unittest.TestCase):
 
     def test_get_current(self):
         """Test getting the currently logged-in user."""
-        self.assertEqual(api.user.get_current().getUserName(), TEST_USER_NAME)
+        self.assertEqual(
+            api.user.get_current().getUserName(),
+            TEST_USER_NAME,
+        )
 
     def test_get_all_users(self):
         """Test getting all users."""
@@ -198,7 +201,10 @@ class TestPloneApiUser(unittest.TestCase):
         )
         users = [user.getUserName() for user in api.user.get_users()]
 
-        self.assertEqual(users, ['chuck', TEST_USER_NAME])
+        self.assertItemsEqual(
+            users,
+            ['chuck', TEST_USER_NAME],
+        )
 
     def test_get_groups_users(self):
         """Test getting all users of a certain group."""
@@ -248,7 +254,7 @@ class TestPloneApiUser(unittest.TestCase):
         with self.assertRaises(InvalidParameterError):
             api.user.delete(
                 username='chuck@norris.org',
-                user=mock.Mock()
+                user=mock.Mock(),
             )
 
         api.user.create(email='chuck@norris.org', password='secret')
@@ -260,12 +266,18 @@ class TestPloneApiUser(unittest.TestCase):
     def test_delete_username(self):
         """test whether the user has been deleted."""
 
-        api.user.create(username='unwanted', password='secret',
-                        email='unwanted@example.org')
+        api.user.create(
+            username='unwanted',
+            password='secret',
+            email='unwanted@example.org',
+        )
         api.user.delete(username='unwanted')
 
-        user = api.user.create(username='steven', password='secret',
-                               email='steven@example.org')
+        user = api.user.create(
+            username='steven',
+            password='secret',
+            email='steven@example.org',
+        )
         api.user.delete(user=user)
 
     def test_is_anonymous(self):
@@ -277,15 +289,18 @@ class TestPloneApiUser(unittest.TestCase):
 
     def test_get_roles_username(self):
         """Test get roles passing a username."""
-        ROLES = ['Reviewer', 'Editor']
+        ROLES = ['Reviewer', 'Editor', ]
         api.user.create(
             username='chuck',
             email='chuck@norris.org',
             password='secret',
-            roles=ROLES
+            roles=ROLES,
+        )
+
+        self.assertItemsEqual(
+            ROLES + ['Authenticated'],
+            api.user.get_roles(username='chuck'),
         )
-        ROLES = set(ROLES + ['Authenticated'])
-        self.assertEqual(ROLES, set(api.user.get_roles(username='chuck')))
 
     def test_get_roles_user(self):
         """Test get roles passing a user."""
@@ -294,10 +309,13 @@ class TestPloneApiUser(unittest.TestCase):
             username='chuck',
             email='chuck@norris.org',
             password='secret',
-            roles=ROLES
+            roles=ROLES,
+        )
+
+        self.assertItemsEqual(
+            ROLES + ['Authenticated'],
+            api.user.get_roles(user=user),
         )
-        ROLES = set(ROLES + ['Authenticated'])
-        self.assertEqual(ROLES, set(api.user.get_roles(user=user)))
 
     def test_get_roles_username_and_user(self):
         """Test get roles passing username and user."""
@@ -306,26 +324,28 @@ class TestPloneApiUser(unittest.TestCase):
             username='chuck',
             email='chuck@norris.org',
             password='secret',
-            roles=ROLES
+            roles=ROLES,
         )
 
         from plone.api.exc import InvalidParameterError
         with self.assertRaises(InvalidParameterError):
             api.user.get_roles(
                 username='chuck',
-                user=user
+                user=user,
             )
 
     def test_get_roles_no_parameters(self):
         """Test get roles without any parameters."""
-        ROLES = set(['Manager', 'Authenticated'])
-        self.assertEqual(ROLES, set(api.user.get_roles()))
+        self.assertItemsEqual(
+            ['Manager', 'Authenticated'],
+            api.user.get_roles(),
+        )
 
     def test_get_permissions_no_parameters(self):
         """Test get_permissions passing no parameters."""
-        self.assertEqual(  # TODO: maybe assertItemsEqual?
-            set(p[0] for p in getPermissions()),
-            set(api.user.get_permissions().keys())
+        self.assertItemsEqual(
+            [p[0] for p in getPermissions()],
+            api.user.get_permissions().keys(),
         )
 
     def test_get_roles_nonexistant_user(self):
@@ -340,7 +360,7 @@ class TestPloneApiUser(unittest.TestCase):
         with api.env.adopt_user(user=nobody):
             self.assertEqual(
                 api.user.get_roles(),
-                ('Anonymous', )
+                ('Anonymous', ),
             )
 
     def test_get_roles_in_context(self):
@@ -364,9 +384,15 @@ class TestPloneApiUser(unittest.TestCase):
             id='document_one',
             title='Document One',
         )
-        api.user.grant_roles(username='chuck', roles=['Editor'], obj=folder)
+        api.user.grant_roles(
+            username='chuck',
+            roles=['Editor'],
+            obj=folder,
+        )
         self.assertIn(
-            'Editor', api.user.get_roles(username='chuck', obj=document))
+            'Editor',
+            api.user.get_roles(username='chuck', obj=document),
+        )
 
     def test_get_roles_local_only(self):
         """Test get local roles for a user on an object"""
@@ -389,7 +415,11 @@ class TestPloneApiUser(unittest.TestCase):
             id='document_one',
             title='Document One',
         )
-        api.user.grant_roles(username='chuck', roles=['Editor'], obj=folder)
+        api.user.grant_roles(
+            username='chuck',
+            roles=['Editor'],
+            obj=folder,
+        )
         self.assertNotIn(
             'Editor',
             api.user.get_roles(username='chuck', obj=document, inherit=False),
@@ -418,26 +448,33 @@ class TestPloneApiUser(unittest.TestCase):
             id='document_one',
             title='Document One',
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document),
             ['Member', 'Reviewer', 'Authenticated'],
         )
-        api.user.grant_roles(username='chuck', roles=['Editor'], obj=folder)
-        self.assertEqual(
+        api.user.grant_roles(
+            username='chuck',
+            roles=['Editor'],
+            obj=folder,
+        )
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document),
             ['Member', 'Reviewer', 'Authenticated', 'Editor'],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document, inherit=False),
             [],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=folder, inherit=False),
             ['Editor'],
         )
         api.group.grant_roles(
-            groupname='foo', roles=['Contributor'], obj=document)
-        self.assertEqual(
+            groupname='foo',
+            roles=['Contributor'],
+            obj=document,
+        )
+        self.assertItemsEqual(
             ['Contributor'],
             api.user.get_roles(username='chuck', obj=document, inherit=False),
         )
@@ -449,14 +486,14 @@ class TestPloneApiUser(unittest.TestCase):
             username='chuck',
             email='chuck@norris.org',
             password='secret',
-            roles=[]
+            roles=[],
         )
 
         from plone.api.exc import InvalidParameterError
         with self.assertRaises(InvalidParameterError):
             api.user.get_permissions(
                 username='chuck',
-                user=user
+                user=user,
             )
 
         PERMISSIONS = {
@@ -469,11 +506,11 @@ class TestPloneApiUser(unittest.TestCase):
         for k, v in PERMISSIONS.items():
             self.assertEqual(
                 v,
-                api.user.get_permissions(username='chuck').get(k, None)
+                api.user.get_permissions(username='chuck').get(k, None),
             )
             self.assertEqual(
                 v,
-                api.user.get_permissions(user=user).get(k, None)
+                api.user.get_permissions(user=user).get(k, None),
             )
 
     def test_get_permissions_nonexistant_user(self):
@@ -515,14 +552,20 @@ class TestPloneApiUser(unittest.TestCase):
         )
 
         for k, v in PERMISSIONS.items():
-            self.assertEqual(v,
-                             api.user.get_permissions(
-                                 username='chuck',
-                                 obj=folder).get(k, None))
-            self.assertEqual(v,
-                             api.user.get_permissions(
-                                 user=user,
-                                 obj=folder).get(k, None))
+            self.assertEqual(
+                v,
+                api.user.get_permissions(
+                    username='chuck',
+                    obj=folder,
+                ).get(k, None),
+            )
+            self.assertEqual(
+                v,
+                api.user.get_permissions(
+                    user=user,
+                    obj=folder,
+                ).get(k, None),
+            )
 
     def test_has_permission_context(self):
         """Test has_permission on some context."""
@@ -551,15 +594,27 @@ class TestPloneApiUser(unittest.TestCase):
         )
         api.content.transition(obj=folder, transition='publish')
 
-        self.assertTrue(api.user.has_permission(
-            'View',
-            user=user,
-            obj=folder)
+        self.assertTrue(
+            api.user.has_permission(
+                'View',
+                user=user,
+                obj=folder,
+            ),
         )
-        self.assertFalse(api.user.has_permission(
-            'Modify portal content',
-            user=user,
-            obj=folder)
+        self.assertFalse(
+            api.user.has_permission(
+                'Modify portal content',
+                user=user,
+                obj=folder,
+            ),
+        )
+
+        self.assertTrue(
+            api.user.has_permission(
+                'View',
+                user=None,
+                obj=folder,
+            ),
         )
 
     def test_grant_roles(self):
@@ -580,11 +635,21 @@ class TestPloneApiUser(unittest.TestCase):
         self.assertIn('Contributor', api.user.get_roles(user=user))
 
         api.user.grant_roles(username='chuck', roles=['Reader', 'Reader'])
-        ROLES = set(
-            ('Editor', 'Contributor', 'Reader', 'Authenticated', 'Member'),
+        ROLES = [
+            'Editor',
+            'Contributor',
+            'Reader',
+            'Authenticated',
+            'Member',
+        ]
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(username='chuck'),
+        )
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(user=user),
         )
-        self.assertEqual(ROLES, set(api.user.get_roles(username='chuck')))
-        self.assertEqual(ROLES, set(api.user.get_roles(user=user)))
 
     def test_grant_roles_username_and_user(self):
         """Test grant roles passing username and user."""
@@ -599,6 +664,10 @@ class TestPloneApiUser(unittest.TestCase):
         with self.assertRaises(MissingParameterError):
             api.user.grant_roles(username=user)
 
+        from plone.api.exc import InvalidParameterError
+        with self.assertRaises(InvalidParameterError):
+            api.user.grant_roles(username='chuck', roles=('Authenticated'))
+
     def test_grant_roles_anonymous(self):
         """Test granting Anonymous role."""
 
@@ -642,23 +711,31 @@ class TestPloneApiUser(unittest.TestCase):
             password='secret',
         )
 
-        api.user.grant_roles(username='chuck', roles=['Reviewer', 'Editor'])
-        api.user.revoke_roles(username='chuck', roles=['Reviewer'])
+        api.user.grant_roles(username='chuck', roles=['Reviewer', 'Editor', ])
+        api.user.revoke_roles(username='chuck', roles=['Reviewer', ])
         self.assertNotIn('Reviewer', api.user.get_roles(username='chuck'))
         self.assertNotIn('Reviewer', api.user.get_roles(user=user))
         self.assertIn('Editor', api.user.get_roles(username='chuck'))
         self.assertIn('Editor', api.user.get_roles(user=user))
 
         api.user.revoke_roles(username='chuck', roles=('Editor',))
-        ROLES = set(('Authenticated', 'Member'))
-        self.assertEqual(ROLES, set(api.user.get_roles(username='chuck')))
-        self.assertEqual(ROLES, set(api.user.get_roles(user=user)))
-        self.assertEqual(
+        ROLES = ['Authenticated', 'Member', ]
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(username='chuck', inherit=False)))
-        self.assertEqual(
+            api.user.get_roles(username='chuck'),
+        )
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(user=user, inherit=False)))
+            api.user.get_roles(user=user),
+        )
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(username='chuck', inherit=False),
+        )
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(user=user, inherit=False),
+        )
 
     def test_revoke_roles_username_and_user(self):
         """Test revoke roles passing username and user."""
@@ -700,14 +777,28 @@ class TestPloneApiUser(unittest.TestCase):
         with self.assertRaises(MissingParameterError):
             api.user.revoke_roles()
 
+    def test_revoke_roles_username(self):
+        """Test revoke roles for username."""
+        api.user.create(
+            username='chuck',
+            email='chuck@norris.org',
+            password='secret',
+        )
+        from plone.api.exc import InvalidParameterError
+        with self.assertRaises(InvalidParameterError):
+            api.user.revoke_roles(username='chuck', roles=['Authenticated', ])
+
     @unittest.skip('Getting the Anonymous user does not work like this.')
     def test_revoke_roles_from_anonymous(self):
         """Test revoking roles from an Anonymous user."""
-        api.user.revoke_roles(username='Anonymous User', roles=['Reviewer'])
-        ROLES = set(['Anonymous', ])
-        self.assertEqual(
+        api.user.revoke_roles(
+            username='Anonymous User',
+            roles=['Reviewer'],
+        )
+        ROLES = ['Anonymous', ]
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(username='Anonymous User'))
+            api.user.get_roles(username='Anonymous User'),
         )
 
     def test_revoke_roles_no_user(self):
@@ -742,12 +833,16 @@ class TestPloneApiUser(unittest.TestCase):
             title='Document One',
         )
 
-        api.user.grant_roles(username='chuck', roles=['Editor'], obj=folder)
+        api.user.grant_roles(
+            username='chuck',
+            roles=['Editor'],
+            obj=folder,
+        )
         self.assertIn(
             'Editor',
             api.user.get_roles(username='chuck', obj=folder),
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             ['Editor'],
             api.user.get_roles(username='chuck', obj=folder, inherit=False),
         )
@@ -759,7 +854,10 @@ class TestPloneApiUser(unittest.TestCase):
             'Editor',
             api.user.get_roles(username='chuck', obj=document),
         )
-        self.assertIn('Editor', api.user.get_roles(user=user, obj=document))
+        self.assertIn(
+            'Editor',
+            api.user.get_roles(user=user, obj=document),
+        )
 
         api.user.grant_roles(
             username='chuck',
@@ -783,19 +881,22 @@ class TestPloneApiUser(unittest.TestCase):
             api.user.get_roles(user=user, obj=document),
         )
 
-        ROLES = set(('Editor', 'Contributor', 'Authenticated', 'Member'))
-        self.assertEqual(
+        ROLES = ['Editor', 'Contributor', 'Authenticated', 'Member', ]
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(username='chuck', obj=folder)),
+            api.user.get_roles(username='chuck', obj=folder),
         )
-        self.assertEqual(ROLES, set(api.user.get_roles(user=user, obj=folder)))
-        self.assertEqual(
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(username='chuck', obj=document)),
+            api.user.get_roles(user=user, obj=folder),
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(user=user, obj=document)),
+            api.user.get_roles(username='chuck', obj=document),
+        )
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(user=user, obj=document),
         )
 
     def test_grant_roles_disregards_adapter(self):
@@ -821,7 +922,7 @@ class TestPloneApiUser(unittest.TestCase):
             password='secret',
         )
 
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=folder),
             ['Member', 'Authenticated'],
         )
@@ -855,16 +956,19 @@ class TestPloneApiUser(unittest.TestCase):
 
         # Assign a local role
         api.user.grant_roles(
-            username='chuck', roles=['Contributor'], obj=folder)
+            username='chuck',
+            roles=['Contributor'],
+            obj=folder,
+        )
         self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=folder),
             ['Member', 'Authenticated', 'Contributor', 'Reviewer'],
         )
 
         # The adapter role is in in the local roles but not persistent
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=folder, inherit=False),
-            ['Contributor', 'Reviewer'],
+            ['Contributor', 'Reviewer', ],
         )
         local_roles = getattr(folder, '__ac_local_roles__', {})
         self.assertEqual(
@@ -872,46 +976,63 @@ class TestPloneApiUser(unittest.TestCase):
             ['Contributor'],
         )
 
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document, inherit=False),
             ['Reviewer'],
         )
         self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document),
-            ['Member', 'Authenticated', 'Contributor', 'Reviewer'],
+            ['Member', 'Authenticated', 'Contributor', 'Reviewer', ],
         )
 
         # add a group and test mix of group and adapter and user-roles
         api.group.create('foo')
-        api.group.grant_roles(groupname='foo', roles=['Contributor'], obj=document)  # noqa
+        api.group.grant_roles(
+            groupname='foo',
+            roles=['Contributor'],
+            obj=document,
+        )
         api.group.add_user(groupname='foo', username='chuck')
 
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document, inherit=False),
-            ['Contributor', 'Reviewer'],
+            ['Contributor', 'Reviewer', ],
         )
         api.group.grant_roles(groupname='foo', roles=['Manager'], obj=folder)
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document, inherit=False),
-            ['Contributor', 'Reviewer'],
+            ['Contributor', 'Reviewer', ],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=document),
-            ['Contributor', 'Reviewer', 'Manager', 'Authenticated', 'Member'],
+            [
+                'Contributor',
+                'Reviewer',
+                'Manager',
+                'Authenticated',
+                'Member',
+            ],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=folder),
-            ['Contributor', 'Reviewer', 'Manager', 'Authenticated', 'Member'],
+            [
+                'Contributor',
+                'Reviewer',
+                'Manager',
+                'Authenticated',
+                'Member',
+            ],
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             api.user.get_roles(username='chuck', obj=folder, inherit=False),
-            ['Contributor', 'Reviewer', 'Manager'],
+            ['Contributor', 'Reviewer', 'Manager', ],
         )
 
         # cleanup
         gsm = getGlobalSiteManager()
         gsm.unregisterAdapter(
-            factory=LocalRoleProvider, provided=ILocalRoleProvider)
+            factory=LocalRoleProvider, provided=ILocalRoleProvider,
+        )
 
     def test_revoke_roles_in_context(self):
         """Test revoke roles."""
@@ -978,27 +1099,36 @@ class TestPloneApiUser(unittest.TestCase):
         )
         self.assertNotIn('Editor', api.user.get_roles(user=user, obj=document))
 
-        ROLES = set(('Authenticated', 'Member'))
-        self.assertEqual(
+        ROLES = ['Authenticated', 'Member', ]
+        self.assertItemsEqual(
             ROLES,
-            set(api.user.get_roles(username='chuck', obj=folder)),
+            api.user.get_roles(username='chuck', obj=folder),
         )
-        self.assertEqual(
-            ROLES, set(api.user.get_roles(user=user, obj=folder)))
-        self.assertEqual(
-            ROLES, set(api.user.get_roles(username='chuck', obj=document)))
-        self.assertEqual(
-            ROLES, set(api.user.get_roles(user=user, obj=document)))
-        self.assertEqual(
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(user=user, obj=folder),
+        )
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(username='chuck', obj=document),
+        )
+        self.assertItemsEqual(
+            ROLES,
+            api.user.get_roles(user=user, obj=document),
+        )
+        self.assertItemsEqual(
             [],
             api.user.get_roles(username='chuck', obj=folder, inherit=False),
         )
-        self.assertEqual(
+        self.assertItemsEqual(
             [],
-            api.user.get_roles(user=user, obj=folder, inherit=False))
-        self.assertEqual(
+            api.user.get_roles(user=user, obj=folder, inherit=False),
+        )
+        self.assertItemsEqual(
             [],
-            api.user.get_roles(username='chuck', obj=document, inherit=False))
-        self.assertEqual(
+            api.user.get_roles(username='chuck', obj=document, inherit=False),
+        )
+        self.assertItemsEqual(
             [],
-            api.user.get_roles(user=user, obj=document, inherit=False))
+            api.user.get_roles(user=user, obj=document, inherit=False),
+        )
